-- 1. Install the PL/PYTHON package on your DB server
-- This has already been done for you on FME Cloud instances.

-- Activate PostGIS and the language for your database (can skip if this is already enabled)
CREATE EXTENSION postgis;
-- CREATE EXTENSION plpythonu;
-- Or (for Python 3.x)
CREATE EXTENSION plpython3u;

-- 2. Create Table
CREATE TABLE bulk_example
(
  parkid INTEGER PRIMARY KEY,
  name TEXT,
  facilities TEXT,
  washrooms TEXT,
  streetnumber TEXT,
  streetname TEXT,
  neighbourhood TEXT,
  hectare REAL,
  geom geometry(Geometry,4326)
);
   
-- 3. Create temporary table as clone of main bulk_example table with an additional column for tracking the database operation
CREATE TABLE bulk_example_temp AS TABLE bulk_example WITH NO DATA;
ALTER TABLE bulk_example_temp
ADD COLUMN db_operation TEXT;

-- 4. Create Python HTTP request function
CREATE OR REPLACE FUNCTION public.send_request(url text, message text)
  RETURNS text AS
$BODY$
import urllib, http.client, urllib.parse

parse_object = urllib.parse.urlparse(args[0])
headers = {"Content-type": "application/json", "Accept": "application/json"}
conn = http.client.HTTPConnection(parse_object.netloc)
conn.request("POST", parse_object.path, args[1], headers)

response = conn.getresponse()
return str(response.status) + ' ' + response.reason
$BODY$
  LANGUAGE 'plpython3u';

-- 5. Create bulk trigger function to handle different types of bulk database operations
CREATE OR REPLACE FUNCTION bulk_function()
  RETURNS trigger AS
$BODY$
DECLARE
	operation text;

BEGIN 
	SELECT db_operation INTO operation FROM bulk_example_temp GROUP BY db_operation;

	IF (operation = 'INSERT') THEN
		EXECUTE 'INSERT INTO bulk_example (SELECT parkid, name, facilities, washrooms, streetnumber, streetname, neighbourhood, hectare, geom FROM bulk_example_temp)';
	ELSIF (operation = 'UPDATE') THEN
		EXECUTE 'UPDATE bulk_example targ SET parkid = source.parkid, name = source.name, facilities = source.facilities, washrooms = source.washrooms, 
    streetnumber = source.streetnumber, streetname = source.streetname, neighbourhood = source.neighbourhood, hectare = source.hectare, geom = source.geom 
    FROM bulk_example_temp source WHERE targ.parkid = source.parkid';	
	ELSIF (operation = 'DELETE') THEN
		EXECUTE 'DELETE FROM bulk_example WHERE parkid IN (SELECT parkid FROM bulk_example_temp)';
	END IF;

	RETURN NULL;
END 
$BODY$
  LANGUAGE 'plpgsql';

-- 6. Create trigger on insert, update and delete
CREATE TRIGGER trg_bulk_example AFTER INSERT 
ON bulk_example_temp FOR EACH STATEMENT EXECUTE PROCEDURE bulk_function();

-- 7. Create function to call bulk_http after writing from workspace
CREATE OR REPLACE FUNCTION bulk_http()
  RETURNS text AS
$BODY$
DECLARE
	message text;

BEGIN
-- Creates a JSON-formatted message containing an id array of all edited rows and the operation performed
SELECT json_object_agg(db_operation, parkids) INTO message
FROM (
    SELECT db_operation, json_agg(parkid order by parkid) parkids
    FROM bulk_example_temp
    GROUP BY db_operation
    ) sub_query;

PERFORM send_request('http://yourFMEServerWebhookURL', message);

RETURN null;

END
$BODY$
  LANGUAGE 'plpgsql';